'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var tslib_es6 = require('./tslib.es6-9240d9d1.js');
var path = _interopDefault(require('path'));
var chalk = _interopDefault(require('chalk'));
var withTM = _interopDefault(require('next-transpile-modules'));

var NEXT_PATH = '/_next';
var NEXT_BUILD_PATH = './.next/';
function encodePath(scriptPath) {
    return scriptPath.replace(/\W/g, '-').replace(/-(js|jsx|ts|tsx)$/, '');
}
module.exports = function withHydrationInitializer(scriptsByPath) {
    return function withHydration(receivedConfig) {
        if (receivedConfig === void 0) { receivedConfig = {}; }
        // Since we use the `DefinePlugin` to inject the config into `ClientScript`,
        // we need to make sure that this package gets transpiled. Probably it would
        // also be possible to configure the loaders/plugin directly here without
        // using this dependency.
        return withTM(['next-client-script'])(tslib_es6.__assign(tslib_es6.__assign({}, receivedConfig), { webpack: function (config, options) {
                var _a;
                var buildId = options.buildId, dev = options.dev, isServer = options.isServer, nextWebpack = options.webpack;
                // By using this folder, we get immutable caching headers
                var PUBLIC_BASE_PATH = "/static/" + buildId + "/client/";
                var clientEntries = {};
                var clientScriptsByPath = {};
                Object.entries(scriptsByPath).forEach(function (_a) {
                    var pagePath = _a[0], scriptPath = _a[1];
                    var publicPath = PUBLIC_BASE_PATH + encodePath(scriptPath);
                    if (dev)
                        publicPath += '.js';
                    clientScriptsByPath[pagePath] = NEXT_PATH + publicPath;
                    if (!dev)
                        clientScriptsByPath[pagePath] += '.js';
                    var entryPointName = publicPath.substring(1);
                    clientEntries[entryPointName] = scriptPath;
                });
                config = tslib_es6.__assign(tslib_es6.__assign({}, config), { plugins: (_a = config.plugins) === null || _a === void 0 ? void 0 : _a.concat(new nextWebpack.DefinePlugin({
                        CLIENT_SCRIPTS_BY_PATH: JSON.stringify(clientScriptsByPath)
                    })) });
                if (isServer) {
                    return config;
                }
                if (dev) {
                    // In development, we can handle the script simply as another entry point
                    var originalEntry_1 = config.entry;
                    config.entry = function () {
                        return originalEntry_1().then(function (entry) { return (tslib_es6.__assign(tslib_es6.__assign({}, entry), clientEntries)); });
                    };
                }
                else {
                    // For the production build, we have to trigger a separate build where
                    // only the client entries are compiled. A child compiler didn't work
                    // during initial tests, as nothing was written to the file system.
                    var clientConfig = tslib_es6.__assign(tslib_es6.__assign({}, config), { entry: clientEntries, output: {
                            path: path.resolve(process.cwd(), NEXT_BUILD_PATH)
                        }, optimization: tslib_es6.__assign(tslib_es6.__assign({}, config.optimization), { 
                            // Output only a single JavaScript asset that
                            // contains all necessary client code.
                            runtimeChunk: undefined }), plugins: (config.plugins || []).filter(function (plugin) {
                            return ![
                                // The manifest from the regular build should be used
                                'ReactLoadablePlugin',
                                'BuildManifestPlugin'
                            ].includes(plugin.constructor.name);
                        }) });
                    var compiler = nextWebpack(clientConfig);
                    compiler.run(function (error, stats) {
                        var errorMessage;
                        if (error) {
                            errorMessage = error.message;
                        }
                        if (stats.compilation.errors.length > 0) {
                            errorMessage = stats.compilation.errors
                                .map(function (error) {
                                var message;
                                if (error.message) {
                                    message = error.message;
                                    if (error.stack) {
                                        message += '\n' + error.stack;
                                    }
                                }
                                else {
                                    try {
                                        message = JSON.stringify(error);
                                    }
                                    catch (error) {
                                        message = 'An unknown error happened.';
                                    }
                                }
                                return message;
                            })
                                .join('\n\n');
                        }
                        if (errorMessage) {
                            console.error(errorMessage);
                            process.exit(1);
                        }
                        /* eslint-disable no-console */
                        console.log(chalk.green('\nCreated client scripts'));
                        Object.entries(scriptsByPath).forEach(function (_a) {
                            var path = _a[0], script = _a[1];
                            console.log(path + ": " + script);
                        });
                        console.log('\n');
                        /* eslint-enable no-console */
                    });
                }
                // Overload the Webpack config if it was already overloaded
                if (typeof receivedConfig.webpack === 'function') {
                    return receivedConfig.webpack(config, options);
                }
                return config;
            } }));
    };
};
//# sourceMappingURL=withClientScripts.js.map
